@@ -28,6 +28,7 @@
 #include <libgnome/gnome-i18n.h>
 #include <libxml/parser.h>
 #include <libxml/parserInternals.h>
+#include <libxml/xinclude.h>
 #include <libxslt/xslt.h>
 #include <libxslt/templates.h>
 #include <libxslt/transform.h>
@@ -45,7 +46,8 @@
 #define DB_STYLESHEET_PATH DATADIR"/sgml/docbook/yelp/"
 #define DB_STYLESHEET      DB_STYLESHEET_PATH"db2html.xsl"
 
-#define MAX_CHUNK_DEPTH 2
+#define BOOK_CHUNK_DEPTH 2
+#define ARTICLE_CHUNK_DEPTH 1
 
 #define d(x)
 
@@ -237,7 +239,6 @@ db_pager_process (YelpPager *pager)
 			   (const char *) path,
 			   NULL,
 			   XML_PARSE_DTDLOAD  |
-			   XML_PARSE_XINCLUDE |
 			   XML_PARSE_NOCDATA  |
 			   XML_PARSE_NOENT    |
 			   XML_PARSE_NONET    );
@@ -248,11 +249,21 @@ db_pager_process (YelpPager *pager)
 	return FALSE;
     }
 
+    xmlXIncludeProcessFlags (doc,
+			     XML_PARSE_DTDLOAD  |
+			     XML_PARSE_NOCDATA  |
+			     XML_PARSE_NOENT    |
+			     XML_PARSE_NONET    );
+
     walker = g_new0 (DBWalker, 1);
     walker->pager     = YELP_DB_PAGER (pager);
     walker->doc       = doc;
     walker->cur       = xmlDocGetRootElement (walker->doc);
-    walker->max_depth = MAX_CHUNK_DEPTH;
+
+    if (!xmlStrcmp (xmlDocGetRootElement (doc)->name, BAD_CAST "book"))
+	walker->max_depth = BOOK_CHUNK_DEPTH;
+    else
+	walker->max_depth = ARTICLE_CHUNK_DEPTH;
 
     id = xmlGetProp (walker->cur, "id");
     if (id)
@@ -326,7 +337,7 @@ db_pager_process (YelpPager *pager)
     xmlFreeDoc (doc);
     xsltFreeStylesheet (stylesheet);
     xmlFreeParserCtxt (ctxt);
-
+    xsltFreeTransformContext (tctxt);
 
     yelp_pager_set_state (pager, YELP_PAGER_STATE_FINISHED);
     g_signal_emit_by_name (pager, "finish");
@@ -505,6 +516,10 @@ xslt_yelp_cache (xsltTransformContextPtr ctxt,
 		 xmlNodePtr              inst,
 		 xsltStylePreCompPtr     comp)
 {
+    xsltApplyOneTemplate (ctxt, node, inst->children, NULL, NULL);
+
+    while (gtk_events_pending ())
+	gtk_main_iteration ();
 }
 
 static void
